#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "gpmi.h"

/* Include package header - for compiling outside of gpmi, this would be
   #include <gpmi_pkg/example.h> */
#include "packages/example.h"

/**
 * An example how to use a package directly from an application
 * Generally it's a bad idea: you won't be able to access package's commands
 * directly, only trough the string-interface because the linker can't
 * find the referred commands.
 *
 * From modules, you can either use the string interface in the same way
 * as shown here, or directly call the c commands.
 */

static char *(*Example_int_str)(int argc, char *argv[]) = NULL;
static int (*Example_int_bin)(int offs) = NULL;

/* I am not sure that printf prints "(nil)" the same way on every platform */
void *nil = NULL;

static void reg_cmd(void *handle, char *name, void *proc_wrap, void *proc_lowlevel, gpmi_package *pkg)
{
	printf("Reg_cmd. handle=%s; name=%s, address=%p\n", (char *)handle, name, proc_wrap);
	if (strcmp(name,"Example_int") == 0) {
		Example_int_str = proc_wrap;
	}
}


void test_packages(gpmi_module *mod_example) {
	char *params[2]={"Example_int", "10"};
	char *result;
	gpmi_package *pkg;
	int i;

	/* Feature: load a package
	   Level:   optional

		 Last param, handle, is optional, you can pass an argument this way
		 to your reg_cmd that will be called for each new command by the package.
	*/

	if (gpmi_pkg_load("example", 0, reg_cmd, "MyHandle", &pkg) != 0)
		return;

	printf("gpmi_pkg_load: %p != %p\n", pkg, nil);

	/* Feature: run a package command - string based
	   Level:   optional

		 You pass it argc/argv and it returns a char * that you will need to free.
	*/
	result = Example_int_str(2, params);
	if (atoi(result) != 1244)
		printf("Example_int_str have failed badly: %s\n", result);
	else
		printf("Example_int_str works: %s == 1244\n", result);
	free(result);

	/* Feature: run a package command - binary interface
	   Level:   optional

		 You need to know the exact prototype of the given routine
		 or the type of the given variable.
	*/
	Example_int_bin = gpmi_pkg_resolve(pkg, "Example_int");
	printf("Example_int_bin ptr %p != %p means ok\n", Example_int_bin, nil);

	i = Example_int_bin(10);
	if (i != 1244)
		printf("Example_int_bin have failed badly: %d is not 1244\n", i);
	else
		printf("Example_int_bin works %d == 1244.\n", i);

	/* Feature: unload a package
	   Level:   optional

		 Any package loaded with gpmi_pkg_load() must be unloaded when it's
		 no longer needed. 
	*/
	gpmi_pkg_unload(pkg);

}

