#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gpmi.h"

/* This function is used to print a detailed error message */
void print_gpmi_error(gpmi_err_stack_t *entry, char *string)
{
	printf("-> %s\n", string);
}

int main(int argc, char *argv[]) {
	gpmi_module *mod_script[128];
	int n, m, modules;

	if (argc < 3) {
		printf("Usage: %s module script\n", argv[0]);
		printf("Example: %s tcl example_tcl\n", argv[0]);
		return 1;
	}

	gpmi_path_append(GPMI_PATH_MODULES, GPMI_ROOT "/modules");
#ifdef WINDLL
	gpmi_path_append(GPMI_PATH_PACKAGES, GPMI_ROOT "/gpmi_pkg");
#else
	gpmi_path_append(GPMI_PATH_PACKAGES, GPMI_ROOT "/packages");
#endif
	gpmi_path_append(GPMI_PATH_SCRIPTS, GPMI_ROOT "/scripts");

	gpmi_init();
	gpmi_err_stack_enable();

	modules = 0;
	for(n = 1; n < argc; n += 2, modules++) {
		if (modules >= 128) {
			printf("Too many modules already!\n");
			exit(1);
		}
		mod_script[modules] = gpmi_mod_load(argv[n], argv[n+1]);
		if (mod_script[modules] == NULL) {
			printf("Couldn't open %s module, stack:\n", argv[n+1]);
			gpmi_err_stack_process_str(print_gpmi_error);
			return 1;
		}
	}

	for(n = 0; n < 256; n++)
		for(m = 0; m < modules; m++)
			gpmi_call_RunTick(mod_script[m], n);


	for(m = 0; m < modules; m++)
		gpmi_call_RunData(mod_script[m], (void *)0x1234);

	for(m = 0; m < modules; m++)
		gpmi_mod_unload(mod_script[m]);

	gpmi_uninit();

	printf("Example finished.\n");

	return 0;
}
