/* WARNING: rehash kills all cursors on a hash table! */
#include <stdlib.h>
#include <assert.h>
#include "hash_cursor.h"

/* Create a new cursor for hash table h */
hash_cursor_t *hash_cursor_new(hash_t *h)
{
	hash_cursor_t *curs;
	assert(h != NULL);
	
	curs = malloc(sizeof(hash_cursor_t));
	curs->h       = h;
	curs->tbl_pos = -1;
	curs->current = NULL;

	return curs;
}

/* Rewind: set cursor to beginning of the table, return the first record's data */
void *hash_cursor_rewind(hash_cursor_t *curs)
{
	curs->tbl_pos = -1;
	curs->current = NULL;
	return hash_cursor_next(curs);
}

/* Move cursor to the next record, return data field of this record */
void *hash_cursor_next(hash_cursor_t *curs)
{
	/* We are inside a linked list already, but not at its end */
	if ((curs->current != NULL) && (curs->current->next != NULL)) {
		curs->current = curs->current->next;
		return (void *)curs->current->data;
	}
	
	/* We need to find the next non-empty slot */
	for(curs->tbl_pos++; curs->tbl_pos < curs->h->tbl_size; curs->tbl_pos++) {
		if (curs->h->tbl[curs->tbl_pos] != NULL) {
			curs->current = (void *)curs->h->tbl[curs->tbl_pos];
			return (void *)curs->current->data;
		}
	}

	/* No more records */
	return NULL;
}

/* Return key field of the current record; call it only when the cursor points to a valid record! */
void *hash_cursor_key(hash_cursor_t *curs)
{
	assert(curs->current != NULL);
	return curs->current->key;
}

/* Return data filed of the current record; call it only when the cursor points to a valid record! */
void *hash_cursor_data(hash_cursor_t *curs)
{
	assert(curs->current != NULL);
	return (void *)curs->current->data;
}

/* Overwrites the data field with new data. Old data is not freed! */
void hash_cursor_data_write(hash_cursor_t *curs, void *new_data)
{
	assert(curs->current != NULL);
	curs->current->data = new_data;
}

/* Destroy cursor, free all memory used by the cursor */
void hash_cursor_destroy(hash_cursor_t *curs)
{
	free(curs);
}
