/*
    hash 1.0.0 simple hash tables with linked lists
    Copyright (C) 2004 Tibor `Igor2` Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Author: igor2gpmi@peticio.hu
*/


/* This file contains code for providing statistics on a hash table */
#include <stdlib.h>
#include <string.h>
#include "hash.h"

int hash_stat(struct hash_t *h, int *recno, int *empty_nodes, float *avg_rec_per_node, int *max_rec_per_node, int *memory)
{
	int rn, en, mrpn, mem;
	float arpn;
	struct hdata_t **a, *b;
	int n, m;

	rn = 0;
	en = 0;
	mrpn = 0;
	mem = 0;

	for (a = h->tbl, n = 0; n < h->tbl_size; n++, a++) {
		b = *a;
		if (b != NULL) {
			for (rn++, m = 1; b != NULL; b = b->next, m++, rn++)
				mem += sizeof(struct hdata_t);
			if (m > mrpn)
				mrpn = m;
			rn--;
		}
		else {
			//no record at this position
			en++;
		}
	}

	if (recno != NULL)
		*recno = rn;

	if (empty_nodes != NULL)
		*empty_nodes = en;

	if (avg_rec_per_node != NULL) {
		arpn = rn;
		arpn = arpn / (h->tbl_size - en);
		*avg_rec_per_node = arpn;
	}

	if (max_rec_per_node != NULL)
		*max_rec_per_node = mrpn;

	if (memory != NULL) {
		mem += h->tbl_size * 4 + sizeof(struct hash_t) + rn * sizeof(struct hdata_t);
		*memory = mem;
	}

	return 0;
}
