#ifndef GPMI_OS_DEP_H
#define GPMI_OS_DEP_H

#ifdef WIN32
char *basename(char *path);
char *dirname(char *path);
#endif

#ifdef WINDLL
#	include <windows.h>
#	include <winbase.h>
#elif defined(DLCOMPAT)
#	include <dl-compat.h>
#else
#	include <dlfcn.h>
#	include <limits.h>
#endif

/* Define a library handle */
#ifdef WINDLL
#	define gpmi_library_handle HMODULE
#else
#	define gpmi_library_handle void *
#endif

#ifndef PATH_MAX
#ifdef MAX_PATH
#define PATH_MAX MAX_PATH
#else
#define PATH_MAX 256
#endif
#endif

#ifdef WIN32
#define PRINTF_PTR "0x%x"
#define PRINTF_PTR_CAST (size_t)
#define GPMI_PATH_SEP_STR ";"
#define GPMI_PATH_SEP_CHAR ';'
#else
#define PRINTF_PTR "%p"
#define PRINTF_PTR_CAST
#define GPMI_PATH_SEP_STR ":"
#define GPMI_PATH_SEP_CHAR ':'
#endif

#ifndef OS_DEP_MACROS_ONLY

/* Functions for the user */
int  gpmi_install_root(char *inst_root, int inst_root_size);

/* Low level functions */
void  gpmi_library_add_path(const char *dir, const char *library_name, char *lib_file_name, char *dep_file_name, int path_size);
int   gpmi_library_load_lowlevel(gpmi_library *library, char *file_name);
void  gpmi_library_unload_lowlevel(gpmi_library *library);
void *gpmi_dlsym_lowlevel(gpmi_library *library, const char *name);
void *gpmi_dlopen_global();

/* Path assembly */
#ifdef WIN32
	char *gpmi_init_path(char *path);
#else
	/* Keep fast way for non-win32 */
	#define gpmi_init_path(path) strdup(GPMI_INST_ROOT path);
#endif

#else
/* Make header-file free to readd */
#	undef GPMI_OS_DEP_H
#endif

#endif /* GPMI_OS_DEP_H */
