#include "tcl.h"
#include <gpmi/debug.h>
#include <gpmi/modules.h>

static const GPMI_Script_Info TCL_Proc_Info = {
	&TCL_Initialize,
	&TCL_Destroy,
	&TCL_GetExtension,
	&TCL_RegisterCommand,
	&TCL_LoadScript,
	&TCL_Rehash,
	&TCL_RunFunction,
	NULL,
	0,
};

void gpmi_mod(RunTick)(gpmi_module *module, int frame_counter)
{
	GPMI_Script_RunTick(module, frame_counter);
}

void gpmi_mod(RunData)(gpmi_module *module, void *data)
{
	GPMI_Script_RunData(module, data);
}

/**
 * Run direct tcl command from the console. For debugging.
 * At this time it runs in the interpreter of the current AI that is
 * fine with the single AI approach. Later we will need to add commands
 * to switch between AIs.
*/
int gpmi_mod(Debug)(gpmi_module *module, const char *parameters)
{
	TCL_ClientData *TCL_MyData = module->mod_data;
	Tcl_Obj *res;

	if ( Tcl_Eval(TCL_MyData->interp, parameters) != TCL_OK )
		TCL_ReportTclError(TCL_MyData);
	else {
		res = Tcl_GetObjResult(TCL_MyData->interp);
		gpmi_error(1, "[TCL] Result: %s\n",  res->bytes);
	}

	return 0;
}

/**
 * Necessarry routines to init/uninit the module
 */
int gpmi_mod(Initialize)(gpmi_module *module, const char *params)
{
	return GPMI_Script_Load(&TCL_Proc_Info, module, params);
}

void gpmi_mod(Uninitialize)(gpmi_module *module)
{
	GPMI_Script_Destroy(module);
}
