//#define debug 5
#include <stdlib.h>
#include <string.h>
#include "filter.h"

#ifdef debug
#include <stdio.h>
static FILE *flt_log = NULL;
#endif

char *flt_ws = " \t\r\n";

void memcpy_w(char *dest, char *src, int len)
{
#ifdef debug
	fprintf(flt_log, "memcpy_w copying %d bytes to %p from %p:  ", len, dest, src);
#endif
	for (; len > 0; len--, dest++, src++) {
#ifdef debug
		fprintf(flt_log, "%d:[%d/'%c'] ", len, *src, *src);
#endif
		*dest = *src;
	}
#ifdef debug
	fprintf(flt_log, "\n");
#endif

}

void flt_change_sep(struct filter_t *f, char *newsep)
{
	if (newsep == NULL)
		f->sep = NULL;
	else {
		f->sep = realloc(f->sep, strlen(newsep) + 5);
		strcpy(f->sep, newsep);
	}
}

void flt_step(struct filter_t *f, int step)
{
	f->step = step;
}


struct filter_t *flt_init(char *sep, int skip, int flags)
{
	struct filter_t *f;
	f = malloc(sizeof(struct filter_t));
	f->sep = NULL;
	flt_change_sep(f, sep);
	f->la = 0;
	f->l = 0;
	f->data = NULL;
	f->next = 0;
	f->last = 0;
	f->skip = skip;
	f->flags = flags;
	f->step = 0;

#ifdef debug
	if (flt_log == NULL)
		flt_log = fopen("filter.log", "w");

	fprintf(flt_log, "%p: INIT '%s' %d %d DV: %d\n", f, sep, skip, flags, debug);
	fflush(flt_log);
#endif
	return f;
}

int flt_fill(struct filter_t *f, char *dt, int len, char **res)
{
	int l, a;
	char *s, *ss;

#ifdef debug
	int n;

	if (dt == NULL)
		fprintf(flt_log, "\n%p: fill (NULL) %d %p\n", f, len, res);
	else
		fprintf(flt_log, "\n%p: fill '%s' %d %p\n", f, dt, len, res);
	fflush(flt_log);
#endif

	if (f->next != 0) {						//if data left from the last read, kill
#ifdef debug
		fprintf(flt_log, "%p: D1 %d\n", f, f->next);
		fflush(flt_log);
#endif
		memmove(f->data, f->data + f->next, f->l - f->next);
		f->last -= f->next;
		f->l -= f->next;
		f->next = 0;
	}
	if (len > 0) {								//merge in new data, if any
#ifdef debug
		fprintf(flt_log, "%p: D2 '%s' %d la=%d l=%d\n", f, dt, len, f->la, f->l);
		fflush(flt_log);
#endif
		a = f->l + len;
		if (a > f->la) {
			f->la = a + f->skip;
			f->data = realloc(f->data, f->la);
		}
		memcpy_w(f->data + f->l, dt, len);
		f->l += len;
	}
#ifdef debug
	fprintf(flt_log, "%p: D2B ", f);
	for (n = 0; n < f->l; n++) {
		fprintf(flt_log, "[%d/'%c']", f->data[n], f->data[n]);
	}
	fprintf(flt_log, "\n");
	fflush(flt_log);
#endif

	if (res == NULL)
		return 0;										//the user does not want to poll

#ifdef debug
	fprintf(flt_log, "%p: D3\n", f);
	fflush(flt_log);
#endif

	if (f->sep == NULL) {					//no separator, stepping 
#ifdef debug
		fprintf(flt_log, "%p: D4\n", f);
		fflush(flt_log);
#endif
		if (f->step == 0) {					//return the whole string
#ifdef debug
			fprintf(flt_log, "%p: D5\n", f);
			fflush(flt_log);
#endif
			f->next = f->l;
			*res = f->data;
			return f->next;
		}
		else if (f->l >= f->step) {	//have enough bytes read
#ifdef debug
			fprintf(flt_log, "%p: D6\n", f);
			fflush(flt_log);
#endif
			f->next = f->step;
			*res = f->data;
			return f->next;
		}
		*res = NULL;
		return 0;										//stepping, not enough bytes yet
	}
#ifdef debug
	fprintf(flt_log, "%p: D7\n", f);
	fflush(flt_log);
#endif
	//seek the next separator - only from the last checked byte 
	for (l = f->l - f->last, s = f->data + f->last; l > 0; l--, s++, f->last++) {
#ifdef debug
		fprintf(flt_log, "%p: D7A l=%d s=%p; *s=%d/'%c' f->last=%d NEXT: %d/'%c'\n", f, l, s, *s, *s, f->last, *(s + 1), *(s + 1));
		fflush(flt_log);
#endif
		for (ss = f->sep; *ss; ss++) {

#ifdef debug
			fprintf(flt_log, "%p: D7B ss=%p **ss=%d\n", l, ss, *ss);
			fflush(flt_log);
#endif

			if (*s == *ss) {
				f->last++;
				f->next = f->last;
				*res = f->data;
				if ((f->flags & flt_oversep) != 0)
					*s = 0;
#ifdef debug
				fprintf(flt_log, "%p: D7C *res='%s'; f->next=%d\n", l, *res, f->next);
				fflush(flt_log);
#endif
				return f->next;
			}
		}
	}
#ifdef debug
	fprintf(flt_log, "%p: D8\n", f);
	fflush(flt_log);
#endif
	*res = NULL;
	return 0;
}

void flt_close(struct filter_t *f)
{
#ifdef debug
	fprintf(flt_log, "%p: CLOSE\n", f);
	fflush(flt_log);
#endif
	if (f->data != NULL)
		free(f->data);
	if (f->sep != NULL)
		free(f->sep);
	f->data = NULL;
	f->la = 0;
	f->l = 0;
	free(f);
}

int flt_peek_skip(struct filter_t *f, char **res)
{
	int n;

	/* No data at all? */
	if (f->data == NULL) {
		if (res != NULL)
			*res = NULL;

		return 0;
	}

	if (res != NULL)
		*res = f->data + f->next;

	n = f->l - f->next;

	if (f->skip == 0)
		return n;

	if (f->skip < n)
		return f->skip;

	return n;

}
