#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "wrapgen.h"
#include "generators.h"
#include "wrapgen_keywords.h"
#include "../gpmi/global.h"

#define max_entries		1024

char *type_declarations[] = {
	"gpmi_keyword",
	"int",
	"char",
	"double",
	"void",
	"unsigned int",
	"void",
	"unsigned char",
	"char",
	NULL
} ;

char *inv_test[] = {
	"== NULL",
	"== gpmi_invalid_int",
	"== NULL",
	"== 0",
	"== NULL",
	"== gpmi_invalid_uint",
	"== NULL",
	"== 0",
	"== 0",
	NULL
} ;

char param[max_entries][64];
char type[max_entries][64];
int type_[max_entries];
int flags[max_entries];

int PrintVarDefs(int *longest_name, int *longest_type, int params, char *cmd, str_buff_t *out_file)
{
	int var_count, n, my_type;
	char buff[number_of_types*2][1024];
	int count[number_of_types*2];

	var_count = 0;

	/* Generate normal variable definitions*/
	for (n = 0; n < number_of_types; n++) {
		count[n] = 0;
		sprintf(buff[n], "\t%s ", type_declarations[n]);
	}

	/* Generate const variable definition*/
	for (; n < 2 * number_of_types; n++) {
		count[n] = 0;
		sprintf(buff[n], "\tconst %s ", type_declarations[n - number_of_types]);
	}

 /* Generate variable name lists */
	for (n = 0; n < params; n++) {

		/* Account lingests for filling in the spaces */
		if (strlen(param[n]) > *longest_name)
			*longest_name = strlen(param[n]);
		if (strlen(type[n]) > *longest_type)
			*longest_type = strlen(type[n]);

		/* Add each type in the right list and check invalid prefixes */
		my_type = type_[n];

		/* Const variables have a different table */
		if ( (flags[n] & flag_const) != 0 )
			my_type += number_of_types;

		switch (type_[n]) {
			case type_void:
				fprintf(stderr, "Warning: it is not a good idea to have a parameter as void, compile will fail. Did you mean ptr?\n");
				break;

			case type_uchar:
			case type_char:
			case type_uint:
			case type_int:
			case type_double:
				/* Simple types */
				if ( (flags[n] & (flag_multiple | flag_dynamic)) != 0 )
					goto bad_prefix;
				strcat(buff[my_type], param[n]);
				strcat(buff[my_type], ", ");
				break;

			case type_string:
			case type_ptr:
				/* Pointer types */
				if ( (flags[n] & flag_multiple) != 0 )
					goto bad_prefix;
				strcat(buff[my_type], "*");
				strcat(buff[my_type], param[n]);
				strcat(buff[my_type], ", ");
				break;

			case type_keyword:
				if ( (flags[n] & flag_dynamic) != 0 )
					goto bad_prefix;

				/* Multiple keywords are resolved directly to integers */
				if ( (flags[n] & flag_multiple) != 0 ) {
					my_type = type_int;
					strcat(buff[my_type], param[n]);
					strcat(buff[my_type], ", ");
				}
				else {
					/* Single keywords are resolved to the type of the keyword table */
					my_type = keyword_find(type[n]);
					if (my_type < 0) {
						fprintf(stderr, "Error: type of keyword table '%s' is not defined!\n", type[n]);
						exit(1);
					}
					strcat(buff[my_type], param[n]);
					strcat(buff[my_type], ", ");
				}
				break;

			default:
				fprintf(stderr, "ERROR: PrintVarDefs can't handle type %s!\n", type_names[type_[n]]);

		}
		var_count++;
		count[my_type]++;
		continue;
		bad_prefix:;
			fprintf(stderr, "PANIC: Invalid type prefix for parameter %s in %s\n", param[n], cmd);
			exit(1);
	}

	/* Print all the variable defs those are not empty */
	for (n = 0; n < number_of_types*2; n++)
		if ((count[n] > 0) && (n != type_void)) {
			buff[n][strlen(buff[n])-2] = 0;
			sbprintf(out_file, "%s;\n", buff[n]);
		}
	return var_count;
}

static int params;

void *WrapgenNewProc()
{
	params = 0;
	return NULL;
}

int WrapgenGotParam(void *current, int line, char *param_str, char *type_str, int my_flags, int my_type)
{
	strcpy(param[params], param_str);
	strcpy(type[params], type_str);
	flags[params] = my_flags;
	type_[params] = my_type;
	params++;
	return params;
}

void Do_Pkg(char *listname) {
	FILE *f;

	/* Generate function wrappers and tcl commands */
	f = fopen(listname, "r");

	if (f == NULL) {
		fprintf(stderr, "Wrapgen: can't open file %s for reading\n", listname);
		return;
	}

	/* Load keyword definitions */
	params = 0;
	GenWrapper(f, NULL, NULL, NULL, NULL, "Keywords");

	/* Generate function wrappers */
	fseek(f, 0, SEEK_SET);
	params = 0;
	GenWrapper(f, GenPkgFunctions, WrapgenNewProc, WrapgenGotParam, GenPkgFuncFiles, "Functions");



	/* Generate event initializers */
	fseek(f, 0, SEEK_SET);
	params = 0;
	GenWrapper(f, GenPkgEvents, WrapgenNewProc, WrapgenGotParam, GenPkgEventsFiles, "Events");

	/* Generate enum initializers */
	fseek(f, 0, SEEK_SET);
	params = 0;
	GenWrapper(f, GenPkgEnums, WrapgenNewProc, WrapgenGotParam, GenPkgEnumsFiles, "Enums");

	fclose(f);

	cfopen(f, gen_prefix "package.c", "w");
	fprintf(f, "/* This file is 'initialization layer 0' for the package: GPMI calls\n   these routines directly and they call other layer 0 functions and\n   layer 1 functions defined in gpmi/package.c\n*/\n\n\n");
	fprintf(f, "#include \"functions.c\"\n");
	fprintf(f, "#include \"events.c\"\n");
	fprintf(f, "#include \"enums.c\"\n\n");

	fprintf(f, "#ifndef gpmi_static\n");
	fprintf(f, "/* DYNAMIC: store our name space */\n");
	fprintf(f, "char *%sdynamic_name_space = \"%s\";\n", PREFIX_GPMI_PKG_GEN, name_space);
	fprintf(f, "#endif\n\n");

	{
		fprintf(f, "/* --- external definitions (by userm gpmi/package.c) --- */\n");
		fprintf(f, "int PKG_FUNC(init)(void);\n");
		fprintf(f, "void PKG_FUNC(uninit)(void);\n");
		fprintf(f, "int PKG_FUNC(checkver)(int requested);\n");
		fprintf(f, "void PKG_FUNC(register)();\n");
		fprintf(f, "\n\n");

		/* Init, uninit, checkver and register */
		fprintf(f, "/* --- init/uninit/checkver/register --- */\n");
		fprintf(f, "int %s%s_init() {\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\t%s%s_enum_init();\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\t%s%s_events_init();\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\treturn PKG_FUNC(init)();\n");
		fprintf(f, "}\n");

		fprintf(f, "void %s%s_uninit() {\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\tPKG_FUNC(uninit)();\n");
		fprintf(f, "\t%s%s_enum_uninit();\n", PREFIX_GPMI_PKG_GEN, name_space);		
		fprintf(f, "}\n\n");

		fprintf(f, "int %s%s_checkver(int requested)\n{\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\treturn PKG_FUNC(checkver)(requested);\n");
		fprintf(f, "}\n\n");

		fprintf(f, "void %s%s_register_commands(void (*regcmd)(void *handle, char *name, void *proc_wrap, void *proc_lowlevel, void *package), void *handle, void *package) {\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "	%s%s_register_commands_(regcmd, handle, package);\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "	PKG_FUNC(register)();\n");
		fprintf(f, "}\n\n\n");

		/* Constructor & destructor */
		fprintf(f, "/* --- static init/uninit --- */\n");
		fprintf(f, "#ifdef gpmi_static\n");

		fprintf(f, "/* STATIC: we need a special init from a constructor (before main()) */\n");
		fprintf(f, "void %s%s_static_init() __attribute__ ((constructor)) ;\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "void %s%s_static_init() {\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\tgpmi_static_pkg_register(FILE_NAME_STR, %s%s_init, %s%s_uninit, %s%s_checkver, %s%s_register_commands);\n", PREFIX_GPMI_PKG_GEN, name_space, PREFIX_GPMI_PKG_GEN, name_space, PREFIX_GPMI_PKG_GEN, name_space, PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "}\n\n");

		fprintf(f, "/* STATIC: we need a special uninit from a destructor (after main()) */\n");
		fprintf(f, "void %s%s_static_uninit() __attribute__ ((destructor)) ;\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "void %s%s_static_uninit() {\n", PREFIX_GPMI_PKG_GEN, name_space);
		fprintf(f, "\tgpmi_static_pkg_unregister(FILE_NAME_STR);\n");
		fprintf(f, "}\n");

		fprintf(f, "#endif\n\n");


	}

	cfclose(f);
}

int main(int argc, char *argv[])
{

	/* Sanitise arguments */
	if (argc != 4) {
		fprintf(stderr, "%s needs exactly three param: WrapperType, ListName, NameSpace\n",argv[0]);
		fprintf(stderr, "  WrapperType tells what kind of wrapper to generate (and it implies output filenames)\n");
		fprintf(stderr, "  ListName donates the source list file to work from\n");
		fprintf(stderr, "  NameSpace is used to generate unique identifiers in some cases\n");
		return 1;
	}

	/* Initialization */
	NOW = time(NULL);
	keyword_init();
	name_space = argv[3];

	if (strcasecmp(argv[1], "Pkg") != 0) {
		fprintf(stderr, "ERROR: wrapgen is called with first parameter different from Pkg.\n");
		fprintf(stderr, "Currently wrapgen can generate only package wrapper, but we reserve\nthe first parameter for later use. If you are generating a package\nwrapper, it's a bug that you don't use Pkg as the first param;\nif you are generating wrapper for something else, this version\nof wrapgen is too new or too old.\n");
		return 1;
	}

	Do_Pkg(argv[2]);

	return 0;
}
